@@ -31,7 +31,7 @@ NULL
 #'
 setLoggingLevel <- function(newLevel = 1L) {
   message("Setting new log level to ", newLevel)
-  oldLevel <- options(COTAN.LogLevel = newLevel)
+  oldLevel <- unlist(options(COTAN.LogLevel = newLevel))
   if (is.null(oldLevel)) {
     oldLevel <- 1L
   }
@@ -203,8 +203,7 @@ handleMultiCore <- function(cores) {
 canUseTorch <- function(optimizeForSpeed, deviceStr) {
   warnedAboutTorch <- !is.null(getOption("COTAN.TorchWarning"))
 
-  useTorch <- isTRUE(optimizeForSpeed) &&
-    requireNamespace("torch", quietly = TRUE)
+  useTorch <- isTRUE(optimizeForSpeed)
 
   if (useTorch) {
     # if torch is not explicitly opted-out, we try using it as
@@ -221,8 +220,7 @@ canUseTorch <- function(optimizeForSpeed, deviceStr) {
     useTorch <- isTRUE(useTorchOpt)
 
     if (!useTorch && !warnedAboutTorch) {
-      warning("The `torch` library is installed,",
-              " but has been explicitly opted out")
+      warning("The `torch` library has been explicitly opted out")
       warning("In case you might try 'options(COTAN.UseTorch = TRUE)'",
               " to enable it")
       warnedAboutTorch <- TRUE
@@ -231,12 +229,12 @@ canUseTorch <- function(optimizeForSpeed, deviceStr) {
 
   if (useTorch) {
     tryCatch({
+      if (!requireNamespace("torch", quietly = TRUE)) {
+        stop("The `torch` library is not installed")
+      }
       if (!torch::torch_is_installed()) {
         stop("The `torch` library is installed but the required",
-             " additional libraries are not avalable yet")
-      }
-      if (!require("torch", character.only = TRUE)) {
-        stop("The `torch` library is installed but cannot be loaded")
+             " additional libraries are not available yet")
       }
       # Call a simple torch function to check if it's working
       if (is.null(torch::torch_tensor(1L))) {
@@ -258,7 +256,7 @@ canUseTorch <- function(optimizeForSpeed, deviceStr) {
   }
 
   if (useTorch) {
-    # Device configuration - fall-back to cpu if no cuda device is available
+    # Device configuration - fall-back to `cpu` if no `cuda` device is available
     if (startsWith(deviceStr, "cuda") && !torch::cuda_is_available()) {
       if (!warnedAboutTorch) {
         warning("The `torch` library could not find any `CUDA` device")
@@ -508,6 +506,66 @@ setColumnInDF <- function(df, colToSet, colName,
   return(df)
 }
 
+#' @details `getMetaInfoRow()` is an internal function: it extracts the row\
+#'   associated with the given tag if present or zero otherwise.
+#'
+#' @param meta The information `data.frame` to update
+#' @param tag The tag associated to the wanted value
+#'
+#' @returns `getMetaInfoRow()` returns the last relevant row position if any or
+#'   zero otherwise.
+#'
+#' @importFrom rlang is_empty
+#'
+#' @importFrom stringr str_equal
+#'
+#' @rdname HandleMetaData
+#'
+getMetaInfoRow <- function(meta, tag) {
+  # nothing to look into
+  if (is_empty(meta)) {
+    return(0L)
+  }
+
+  #get all matches
+  matches <- str_equal(tag, meta[[1L]], ignore_case = TRUE)
+  if (all(!matches)) {
+    return(0L)
+  }
+
+  # get last match!
+  rowPos <- which(matches)
+  rowPos <- rowPos[[length(rowPos)]]
+  return(rowPos)
+}
+
+
+#' @details `updateMetaInfo()` is an internal function: updates an information
+#'   `data.frame`
+#'
+#' @param meta The information `data.frame` to update
+#' @param tag The tag associated to the wanted value
+#' @param value The value or the values to associate to the tag
+#'
+#' @returns `updateMetaInfo()` returns the updated `data.frame`
+#'
+#' @rdname HandleMetaData
+#'
+updateMetaInfo <- function(meta, tag, value) {
+  # get relevant row or a new one if necessary
+  rowPos <- getMetaInfoRow(meta, tag)
+  if (rowPos == 0L) {
+    # new tag: add a new entry
+    rowPos <- nrow(meta) + 1L
+  }
+
+  # all values are converted to strings
+  newLine <- c(tag, paste0(value))
+  meta[rowPos, seq_along(newLine)] <- newLine
+
+  return(meta)
+}
+
 
 #------------------- clusters utilities ----------
 
@@ -896,12 +954,21 @@ plotTheme <- function(plotKind = "common", textSize = 14L) {
   }
 
   if (plotKind == "genes") {
-    return(basicTheme +
-           theme(plot.title = element_text(size = 20L, hjust = 0.02,
-                                           vjust = -10.0, face = "italic",
-                                           color = myDarkBlue),
-                 plot.subtitle = element_text(vjust = -15.0, hjust = 0.01,
-                                              color = "darkred")))
+    return(
+      basicTheme +
+        theme(
+          plot.title.position = "plot",  # title sits above the panel
+          plot.title = element_text(
+            size = 20L,
+            hjust = 0.02,
+            vjust = 1.0,                 # no negative vjust
+            face = "italic",
+            color = myDarkBlue
+            #margin = margin(b = 8)       # spacing between title and panel
+          )
+          #plot.margin = margin(t = 8, r = 6, b = 6, l = 6)
+        )
+    )
   }
 
   if (plotKind == "UDE") {
@@ -963,13 +1030,13 @@ plotTheme <- function(plotKind = "common", textSize = 14L) {
   return(basicTheme)
 }
 
-#' @title getColorsVector
+#' @title `getColorsVector`
 #'
 #' @description This function returns a list of colors based on the
-#'   [brewer.pal()] function
+#'   [RColorBrewer::brewer.pal()] function
 #'
-#' @details The colors are taken from the [brewer.pal.info()] sets with
-#'   `Set1`, `Set2`, `Set3` placed first.
+#' @details The colors are taken from the [RColorBrewer::brewer.pal.info()] sets
+#'   with `Set1`, `Set2`, `Set3` placed first.
 #'
 #' @param numNeededColors The number of returned colors. If omitted it returns
 #'   all available colors
